import { Component, OnInit } from '@angular/core';
import { PageBase } from 'src/app/core/base/page-base';
import { ToastService } from 'src/app/service/toast.service';
import { ConfirmationDialogService } from 'src/app/service/confirmation-dialog.service';
import { PermissionsEnum } from 'src/app/model/auth/permissionsEnum';
import { ShortUrl } from 'src/app/model/entities/short-url';
import { ShortUrlsDataService } from 'src/app/modules/admin/short-urls/short-urls.data.service';
import { ShortUrlsColumns } from 'src/app/modules/admin/short-urls/short-urls.columns';
import { AuthService } from 'src/app/service/auth.service';
import { Filter } from 'src/app/model/graphql/filter/filter.model';

@Component({
  selector: 'app-short-urls',
  templateUrl: './short-urls.page.html',
  styleUrl: './short-urls.page.scss',
})
export class ShortUrlsPage
  extends PageBase<ShortUrl, ShortUrlsDataService, ShortUrlsColumns>
  implements OnInit
{
  shortUrlsWithoutGroup: ShortUrl[] = [];
  groupedShortUrls: { [key: string]: ShortUrl[] } = {};

  get groupsFromGroupedShortUrls() {
    return Object.keys(this.groupedShortUrls);
  }

  private hide_deleted_filter: Filter = { deleted: { equal: false } };
  get showDeleted() {
    return !this.filter.some(
      f => JSON.stringify(f) === JSON.stringify(this.hide_deleted_filter)
    );
  }

  protected hasPermissions = {
    read: false,
    create: false,
    update: false,
    delete: false,
    restore: false,
  };

  constructor(
    private toast: ToastService,
    private confirmation: ConfirmationDialogService,
    private auth: AuthService
  ) {
    super(true, {
      read: [PermissionsEnum.shortUrls],
      create: [PermissionsEnum.shortUrlsCreate],
      update: [PermissionsEnum.shortUrlsUpdate],
      delete: [PermissionsEnum.shortUrlsDelete],
      restore: [PermissionsEnum.shortUrlsDelete],
    });
  }

  async ngOnInit() {
    this.hasPermissions = {
      read: await this.auth.hasAnyPermissionLazy(
        this.requiredPermissions.read ?? []
      ),
      create: await this.auth.hasAnyPermissionLazy(
        this.requiredPermissions.create ?? []
      ),
      update: await this.auth.hasAnyPermissionLazy(
        this.requiredPermissions.update ?? []
      ),
      delete: await this.auth.hasAnyPermissionLazy(
        this.requiredPermissions.delete ?? []
      ),
      restore: await this.auth.hasAnyPermissionLazy(
        this.requiredPermissions.restore ?? []
      ),
    };
  }

  load(): void {
    this.loading = true;
    this.dataService
      .load(this.filter, this.sort, this.skip, this.take)
      .subscribe(result => {
        this.result = result;
        this.shortUrlsWithoutGroup = this.getShortUrlsWithoutGroup();
        this.groupedShortUrls = this.getShortUrlsWithGroup();
        this.loading = false;
      });
  }

  delete(group: ShortUrl): void {
    this.confirmation.confirmDialog({
      header: 'dialog.delete.header',
      message: 'dialog.delete.message',
      accept: () => {
        this.loading = true;
        this.dataService.delete(group).subscribe(() => {
          this.toast.success('action.deleted');
          this.load();
        });
      },
      messageParams: { entity: group.shortUrl },
    });
  }

  restore(group: ShortUrl): void {
    this.confirmation.confirmDialog({
      header: 'dialog.restore.header',
      message: 'dialog.restore.message',
      accept: () => {
        this.loading = true;
        this.dataService.restore(group).subscribe(() => {
          this.toast.success('action.restored');
          this.load();
        });
      },
      messageParams: { entity: group.shortUrl },
    });
  }

  toggleShowDeleted() {
    if (!this.showDeleted) {
      this.filter.splice(this.filter.indexOf(this.hide_deleted_filter), 1);
    } else {
      this.filter.push(this.hide_deleted_filter);
    }

    this.load();
  }

  resetSort() {
    this.sort = [];
    this.load();
  }

  resetFilters() {
    this.filter = [];
    this.load();
  }

  open(url: string) {
    window.open(url, '_blank');
  }

  copy(val: string) {
    navigator.clipboard.writeText(`${window.origin}/${val}`).then(() => {
      this.toast.info('common.copied', 'common.copied_to_clipboard');
    });
  }

  getShortUrlsWithoutGroup(): ShortUrl[] {
    return this.result.nodes.filter(shortUrl => !shortUrl.group);
  }

  getShortUrlsWithGroup() {
    const groupedShortUrls: { [key: string]: ShortUrl[] } = {};

    this.result.nodes.forEach(shortUrl => {
      if (!shortUrl.group) return;

      const groupName = shortUrl.group.name;
      if (!groupedShortUrls[groupName]) {
        groupedShortUrls[groupName] = [];
      }
      groupedShortUrls[groupName].push(shortUrl);
    });

    return groupedShortUrls;
  }
}
